//
//  File:   big_vga_hexdisp3.v
//  Date:   08-Nov-05
//  Author: I. Chuang <ichuang@mit.edu>
//
//  VGA display of string of hex digits
//
//  This version displays 2x2 pixels for each character pixel
//  so we get characters twice the original size.
//
//  11-Nov-05 ILC: now uses a font rom with only 0-9 and A-F - saves space!

module big_vga_hexdisp3 (vclock,pix_clk,hcount,vcount,pixel,data,cx,cy);

   parameter NDIGITS = 8;       // number of 4-bit hex digits to display
   parameter N_BITS = 3; 	// number of bits in NCHAR

   input vclock;	// system synchronous clock
   input pix_clk;	// video pixel clock
   input [9:0] hcount;	// horizontal index of current pixel (0..639)
   input [9:0] vcount;  // vertical index of current pixel (0..479)
   output [2:0] pixel;	// char display's pixel
   input [NDIGITS*4-1:0] data;	// character string to display
   input [9:0] 		 cx;
   input [9:0] 		 cy;

   // 1 line x 8 character display (12 x 24 pixel-sized characters)

   reg [N_BITS-1:0] column;  	// which char in string to display
   wire [9:0] 	    hoff = hcount-cx;
   wire [9:0] 	    voff = vcount-cy;
   reg [4:0] 	    h;	        // 0 .. 23
   reg [5:0] 	    v;		// 0 .. 47

   always @(posedge vclock) 
     begin
	h <= ~pix_clk ? h : ( (hoff==0 | h==0) ? 23 : h-1 );
	v <= ~pix_clk ? v : voff[5:0];
	column <= ~pix_clk ? column 
		  : (hoff==0) ? (NDIGITS-1)
		  : (h==0 ? column - 1 : column);
     end

   // look up hex value to display (from data)

   reg [3:0]  hexdig; 
   integer  n;
   always @(*) 
     for (n=0 ; n<4 ; n = n+1 )		// 4 bits per digit
       hexdig[n] <= data[column*4+n];

   // look up raster row from hex digit font rom
   wire [8:0] font_addr = hexdig*24 + v[5:1];  // 24 words per character
   wire [11:0] font_byte;			// each word has 12 bits
   font1224_hex_rom f(font_addr,vclock,font_byte);

   // generate character pixel if we're in the right h,v area
   wire [2:0] cpixel = font_byte[h[4:1]] ? 7 : 0;
   wire dispflag = ((hcount > cx) & (vcount >= cy) & (hcount <= cx+NDIGITS*24)
		    & (vcount < cy + 48));

   // for debugging
   wire [2:0] bpixel = ((hcount==cx+1|vcount==cy|hcount==cx+NDIGITS*12|vcount==cy+23) ? 3'b010 : 0) | (hcount==cx+1+12 ? 3'b001 : 0)| (hcount==cx+1+24 ? 3'b001 : 0);
//   wire [2:0] pixel = dispflag ? cpixel : 0;

   // latch in pixel, and don't change except on pix_clk 
   reg [2:0] pixel;
   always @(posedge vclock)
//     pixel <= pix_clk ? (dispflag ? cpixel | bpixel : 0) : pixel;
     pixel <= pix_clk ? (dispflag ? cpixel : 0) : pixel;

endmodule
